/**************************************************************************************
* *Copyright (c) 2006 Regents of the University of Chang Gung 						*
* *All rights reserved.													*
 *																*
 * Redistribution and use in source and binary forms, with or without						*
 * modification, are permitted provided that the following conditions					*
 * are met: 															*
 * 1. Redistributions of source code must retain the above copyright						*
 *    notice, this list of conditions and the following disclaimer.						*
 * 2. Redistributions in binary form must reproduce the above copyright					*
 *    notice, this list of conditions and the following disclaimer in the						*
 *    documentation and/or other materials provided with the distribution.					*
 * 3. All advertising materials mentioning features or use of this software					*
 *    must display the following acknowledgement:									*
 *	This product includes software developed by the Computer Systems					*
 *	Engineering Group at Lawrence Berkeley Laboratory.							*
 * 4. Neither the name of the University nor of the Laboratory may be used					*
 *    to endorse or promote products derived from this software without					*
 *    specific prior written permission.										*
 *5. If you have any problem about these codes, 									*
       please mail to antibanish@gmail.com or b9229008@stmail.cgu.edu.tw                    			*
**************************************************************************************/
#ifndef ns_mac_80216_h
#define ns_mac_80216_h
#include "timer-802_16.h"
#include "ip.h"
#include "address.h"
#include "marshall.h"

#include "agent.h"
#include "tclcl.h"
#include "packet.h"
#include "cmu-trace.h"
#include "basetrace.h"
#include "connector.h" 

enum ModulCoding_t { QPSK_12=0, QPSK_34=1, QAM16_12=2, QAM16_34=3, QAM64_23=4, QAM64_34=5};

struct SFID_List
{
	SFID_List()
	{
		SFID = -1;
		TCID = -1;
		BW = -1;
		destIP = -1;
		next = NULL;
	}

	int SFID;
	int TCID;
	int BW;
	int destIP;
	int BCID; //Jessy
	
	SFID_List* next;
};

struct BCID_List
{
	BCID_List()
	{
		BCID = -1; 
		SS_UL_BW = -1;
		SS_DL_BW = -1;
		MC_t = QPSK_12;
		srcIP = -1;
		SF = NULL;
		SF_tail = SF;
		next = NULL;
	}
	
	int BCID;
	int srcIP;
	int SS_UL_BW;
	int SS_DL_BW;
	
	ModulCoding_t MC_t;
	
	SFID_List* SF;
	SFID_List* SF_tail;
	BCID_List* next;
};

struct Queue_construct
{
	Queue_construct():destIP(-5),BCID(-1){}	
	
	SerQueue UGS;
	SerQueue ertPS;
	SerQueue rtPS;
	SerQueue nrtPS;
	SerQueue BE;
	SerQueue PDUQueue;
	SerQueue fragQueue;
	
	int destIP;
	int BCID;
	
	Queue_construct* next_;
};
struct Queue_List
{
	Queue_construct* head_;
	Queue_construct* tail_;
	Queue_List():head_(0),tail_(0){}
	
	void enque(int destip)
	{
		Queue_construct *p=new Queue_construct;
		p->destIP=destip;		
		if (!tail_) 
			head_= tail_= p;
		else 
		{
			tail_->next_= p;
			tail_= p;
		}
		
		tail_->next_= 0;				
	}
	Queue_construct* lookup(int DESTIP) 
	{
		for (Queue_construct* p = head_; p != 0; p = p->next_) 
		{
			if (p->destIP==DESTIP)
				return (p);
		}
		return 0;
	}
	
	Queue_construct* Search_BCID(int bcid) 
	{
		for (Queue_construct* p = head_; p != 0; p = p->next_)
		{
			if (p->BCID==bcid)
				return (p);
		}
		return 0;
	}		
};

enum dir_t { UL=0, DL= 1};
enum MacStatus {MSEND = 0, MIDLE = 1};

class Mac802_16 : public Mac
{
	friend class	TX16timer;
	friend class 	RX16timer;
	friend class 	Defer16Timer;
	friend class 	ULmap16timer;
	friend class	DLmap16timer;
	friend class	Frame;
	friend class 	Preamble;
	friend class 	Backofftimer;
	friend class    ResumeSStimer;
	friend class	ResumeBStimer;
	friend class	SendULmap16timer;
	friend class	SendDLmap16timer;
	friend class	BWBackofftimer;
	friend class	SendDCDtimer;
	friend class	SendUCDtimer;	
	
	public:
		Mac802_16();
		void		sendDCD();
		void		sendUCD();		
		void		sendRNGREQ(int);	
		void		sendRNGRSP(int ssmac,int BSCID);	
		void		sendBWREQ(int BR, int CID, packet_t pype, int DA, int SA);
		bool		Call_Admission_Control(int BCID,packet_t ptype,int BR_BW);
		void		Bandwidth_Management();		
		void		sendDLMAP();
		void		sendULMAP();
		void		BS_Scheduler(int BCID, int Symbol_number);
		void		SS_Scheduler(int BCID, int Symbol_number);
		void		BS_PDU_Generator(packet_t ser_type, int grab,int bcid);
		void 		SS_PDU_Generator(packet_t ser_type, int grab_byte,int bcid);	
		void    	BS_Frag_or_Pack(packet_t ser_type, int num, int usr);
		void 		SS_Frag_or_Pack(packet_t ser_type, int num, int user);			
		void		BS_PDU_transmit(int user, int bw, int Symbol_number);
		void		SS_PDU_transmit(int user, int bw, int Symbol_number);
		void 		transmit(Packet*,double);
		Packet*     Add_Mac_Header(Packet* p,int CID,packet_t ptype);	
		int			Insert_BCID(int srcIP);
		int			Insert_SFID(int BCID,packet_t ptype,dir_t direction_);
		BCID_List*	Search_BCID(int BCID);
		BCID_List*  Search_BCID_macSA(int srcIP);
		SFID_List*	Search_TCID(int TCID);						
		SFID_List* 	Search_SFID(int SFID);
		Queue_construct* Search_Queue(int destIP);
		
	protected:
		int			creatid;		//Node ID, used to identify nodes
		double 		txtime(Packet *p);		//pҶǰeʥ]ɶ	
		Packet* 	pktRNGREQ_;
		Packet* 	pktBr_;
		Packet* 	pktRNGRSP_;
		Packet* 	pktULMAP_;
		Packet* 	pktDLMAP_;
		Packet* 	pktBurst_;
		Packet* 	pktBSBurst_;
		Packet*     pktUCD_;
		Packet*     pktDCD_;
		
	private:
		int 		cw_;		//contention window for rng
		int			cw_bw;
		int			PDU_lock;
		double 		modulation_;
		double		codeRate_;
		double		channelSize_;
		MacState	rx_state_;	// incoming state (MAC_RECV or MAC_IDLE)
		MacState	tx_state_;	// outgoint state
		int			command(int argc, const char*const* argv);
		void 		recv(Packet *, Handler *);	
		
		inline void	mac_log(Packet *p) 
		{
            logtarget_->recv(p, (Handler*) 0);
        }
		
		inline void inc_cw() 
		{
			cw_ = (cw_ << 1) + 1;
			if(cw_ > CW_MAX)
				cw_ = CW_MAX;
		}
		
		inline void inc_cw_bw() 
		{
			cw_bw = (cw_bw << 1) + 1;
			if(cw_bw > CW_MAX)
				cw_bw = CW_MAX;
		}		
		
		void 		checkBackoffTimer();
	
		NsObject*	logtarget_;
		Frame		FrameTimer;
		Preamble 	PreambleTimer;
		ULmap16timer ulmaptimer;
		DLmap16timer dlmaptimer;
		TX16timer 	mhSend_;
		RX16timer 	mhRecv_;
		Defer16Timer mhDefer_;
		Backofftimer BFt;
		ResumeSStimer resumeSSBurst;
		ResumeBStimer resumeBSBurst;
		SendULmap16timer SendUL;
		SendDLmap16timer SendDL;
		BWBackofftimer	 SendBW;	
		SendDCDtimer SendDCD_;
		SendUCDtimer SendUCD_;		
		
		void FrameHandler();
		void PreambleHandler();
		void ULmapHandler(int BCID, int Symbol_number);
		void DLmapHandler(double time, int DL_burst_num, int Symbol_offset);
		void TX16timerHandler();
		void RX16timerHandler();
		void Defer16TimerHandler();
		void BFhandler();
		void ResumeSS16timerHandler(int user, int pkt_count, double interval);
		void ResumeBS16timerHandler(int user, int pkt_count, double interval);
		void SendULmapHandler();
		void SendDLmapHandler();
		void BWhandler();
		void SendDCDHandler();
		void SendUCDHandler();		
};

class Info
{
	public:
		Info():enable(0),macSA(-5),response(0),next_(0){}
		bool enable;
		int macSA;
		int response;
		
		Info* next_;		
};

class Lock
{
	private:					
		Info* head_;
		Info* tail_;
	public:
		Lock() : head_(0), tail_(0){}
		
		int check(int macSA_)
		{			
			for (Info* tmp = head_; tmp != 0; tmp = tmp->next_) 
			{
				if (tmp->macSA==macSA_)
					return (tmp->enable==1)? 2:1;
			}
			return 0;
		}
		
		void unlock(int macSA_)
		{
			for (Info* tmp = head_; tmp != 0; tmp = tmp->next_)
				if (tmp->macSA==macSA_) tmp->enable=1;	
		}
		
		void enque(int macSA_) 
		{
			Info* p=new Info;
			p->macSA=macSA_;
			if (!tail_) 
				head_= tail_= p;
			else 
			{
				tail_->next_= p;
				tail_= p;				
			}
			tail_->next_= 0;				
		}	
		
		int RSPenable(int macSA_)
		{
			for (Info* tmp = head_; tmp != 0; tmp = tmp->next_) 
			{
				if (tmp->response==1)
					return 1;
			}
			return 0;
		}	
		
		void set_response(int macSA_)
		{
			for (Info* tmp = head_; tmp != 0; tmp = tmp->next_) 
				if (tmp->macSA==macSA_)
				{
					tmp->response=1;
					break;	
				}
		}
};
#endif 
