/**************************************************************************************
* *Copyright (c) 2006 Regents of the University of Chang Gung 						*
* *All rights reserved.													*
 *																*
 * Redistribution and use in source and binary forms, with or without						*
 * modification, are permitted provided that the following conditions					*
 * are met: 															*
 * 1. Redistributions of source code must retain the above copyright						*
 *    notice, this list of conditions and the following disclaimer.						*
 * 2. Redistributions in binary form must reproduce the above copyright					*
 *    notice, this list of conditions and the following disclaimer in the						*
 *    documentation and/or other materials provided with the distribution.					*
 * 3. All advertising materials mentioning features or use of this software					*
 *    must display the following acknowledgement:									*
 *	This product includes software developed by the Computer Systems					*
 *	Engineering Group at Lawrence Berkeley Laboratory.							*
 * 4. Neither the name of the University nor of the Laboratory may be used					*
 *    to endorse or promote products derived from this software without					*
 *    specific prior written permission.										*
 *5. If you have any problem about these codes, 									*
       please mail to antibanish@gmail.com or b9229008@stmail.cgu.edu.tw                    			*
**************************************************************************************/
#ifndef __mac_timers_80216_h__
#define __mac_timers_80216_h__

#include <scheduler.h>
#include <assert.h>
//OFDMA Symbol Time
#define OFDMA_Symbol_Time 0.00010084
//Frame Time
#define OFDMA_Frame_Length 0.005
//Number of OFDMA symbol per frame
#define OFDMA_Symbol_per_Frame 49
//Number of OFDMA subchannel
#define OFDMA_subchannel_DLSubFrame 30
#define OFDMA_subchannel_ULSubFrame 35
#define OFDMA_symbol_DLSubFrame 36
#define OFDMA_symbol_ULSubFrame 12
//UܪBandwidthȬȭq std. pp.813
#define OFDMA_QPSK_1_2 4.99
#define OFDMA_QPSK_3_4 7.48
#define OFDMA_16QAM_1_2 9.97
#define OFDMA_16QAM_3_4 14.96
#define OFDMA_64QAM_2_3 19.95
#define OFDMA_64QAM_3_4 22.44
//data payload
#define byte_QPSK_1_2 6
#define byte_QPSK_3_4 9
#define byte_16QAM_1_2 12
#define byte_16QAM_3_4 18
#define byte_64QAM_2_3 24
#define byte_64QAM_3_4 27

#define dcdperiod 10		//NӦvariable
#define ucdperiod 10		//NӦvariable
#define DLmapPeriod OFDMA_Symbol_Time
#define ULmapPeriod OFDMA_Symbol_Time*2
#define DCDfreq 5
#define slottime 0.00001
#define frametime 0.005
#define DLframe 0.003
#define rangingslot 10

#define BR_MAX = 65536
#define DL_Time 10	//10s
#define TTG	0.0002	//0.2ms
#define	Ranging	5	//5s
#define symbol 0.00010084		
#define minslot	8*symbol	
#define OpportunityTime minslot*8
#define OppsOfFrame 10
#define FrameTime 240*symbol //OFDM_Frame=49symbols
#define CW_MAX	1024
#define CW_MIN 1
//Opportunity=8minslot=24symbol




#define T1 50
//T1 maximum = 5*DCD interval maximum value, and DCD interval maximum value = 10s

#define T2 10
//T2 maximum = 5*ranging interval, and ranging interval = 2s

#define T3 0.2
//T3 ranging responsed reception timeout following the transmission of a Ranging Request

#define T4 35
//T4 wait for ranging opportunity   max:35s min:30s

#define T5 2
//T5 wait for Uplink Channel Change response

#define T6 3
//T6 wait for registration response

#define T7 1
//T7 wait for DSA/DSC/DSD Response time out

#define T8 0.003
//T8 wait for DSA/DSC Acknowledge time out

#define T9 0.003
//T9 registration time out 

#define T10 3
//T10 wait for Transaction End timeout

#define T12 50
//T12 wait for DCD Descriptor

#define T13 900
//T13 The time allowed for an SS,following receipt of a REG-RSPmessage to send a TFTP-CPLTmessage to the BS

#define T14 0.002
//Wait for DSX-RVD Timeout

#define T15 0.0002
//Wait for MCA-RSP

#define T16 0.0001
//Wait for bandwidth request grant (QoS dependence)

#define T17 300
//Time allowed for SS to complete SS Authorization and Key Exchange

#define T18 0.0005
//Wait for SBC-RSP timeout  and must smaller than T9

// T19  not specific in Std

#define T20 0.00002
//Time the SS searches for preambles on a given channel   min : 2*Mac Frame

#define T21 10
//Time the SS searches for DL-MAP on a given channel

#define T22 0.5
//Wait for ARQ-Reset

#define T23 1
//Network Entry: Detect network

#define T24 120
//Network Entry: Accumulate MSH-NCFG messages

#define T25 1
//Network Entry: Wait for MSH-NENT/MSH-NCFG

#define T26 0.002
//Wait for TFTP-RSP 			min:10ms default:200 ms max:200 ms


class Mac802_16;

class Mac802_16Timer : public Handler
{
	public:
		Mac802_16Timer(Mac802_16* m):mac(m),rtime(0){busy_ = paused_ = 0; stime = 0.0;};
		virtual void	handle(Event *e)=0;		
		virtual void	start(double time);
		virtual void	stop(void);
		virtual void	pause(void) {assert(0);}	//to be overloaded in subclass
		virtual void	resume(void) {assert(0);}	//to be overloaded in subclass
		inline int	busy(void) {return busy_;}
		inline int	paused(void) {return paused_;}

	protected:
		Mac802_16	*mac;
		int			busy_;
		int			paused_;
		Event		event;
		Event		intr;		
		double		stime;		//start time
		double		wtime;		//waiting time
		double      rtime;
};

//BackoffTime
class Backofftimer : public Mac802_16Timer
{
	private:
		double latence;	
	public:
		Backofftimer(Mac802_16* m):Mac802_16Timer(m),latence(0.0){};
		void 	start(int cw,int idle,double symbolDuration_);
		void	handle(Event *e);
		void	resume(double tmp);
		void	pause(double symbolDuration);
};
class BWBackofftimer : public Mac802_16Timer
{
	private:
		double latence;	
	public:
		BWBackofftimer(Mac802_16* m):Mac802_16Timer(m),latence(0.0){};
		void 	start(int cw,int idle,double symbolDuration_);
		void	handle(Event *e);
		void	resume(double tmp);
		void	pause(double symbolDuration);
};

class TransDATAtimer : public Mac802_16Timer
{
	public:
		TransDATAtimer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double z);
		void 	stop();
		void	handle(Event *);
};

class Frame:public Mac802_16Timer{
	public:
		Frame(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time);		
		void	handle(Event *e);
};

class Preamble:public Mac802_16Timer{
	public:
		Preamble(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time);		
		void	handle(Event *e);
};
class TX16timer:public Mac802_16Timer{
	public:
		TX16timer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time);		
		void	handle(Event *e);
};
class RX16timer:public Mac802_16Timer{
	public:
		RX16timer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time);		
		void	handle(Event *e);
};

class Defer16Timer:public Mac802_16Timer{
	public:
		Defer16Timer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time);		
		void	handle(Event *e);
};

class ULmap16timer:public Mac802_16Timer{
	public:
		int 	BCID_;
		int		Symbol_number_;
		ULmap16timer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time, int BCID, int Symbol_number);		
		void	handle(Event *e);
};

class DLmap16timer:public Mac802_16Timer{
	public:
		int 	DL_burst_num_;
		int		Symbol_offset_;
		double 	time_;
		DLmap16timer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time, int DL_burst_num, int Symbol_offset);
		void	stop();
		void	handle(Event *e);
};

class ResumeSStimer:public Mac802_16Timer{
	public:
		ResumeSStimer(Mac802_16* m):Mac802_16Timer(m){};
		int 	bcid_;
		int     pkt_count_;
		double  interval_;
		void 	start(double time,int usr, int pkt_count);	
		void 	stop();	
		void	handle(Event *e);
};
class ResumeBStimer:public Mac802_16Timer{
	public:
		ResumeBStimer(Mac802_16* m):Mac802_16Timer(m){};
		int 	bcid_;
		int     pkt_count_;
		double  interval_;
		void 	start(double time,int usr,int pkt_count);	
		void 	stop();	
		void	handle(Event *e);
};
class SendULmap16timer:public Mac802_16Timer{
	public:
		SendULmap16timer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double time);	
		void 	stop();	
		void	handle(Event *e);
};
class SendDLmap16timer:public Mac802_16Timer{
	public:
		SendDLmap16timer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double);	
		void 	stop();	
		void	handle(Event *e);
};
class SendDCDtimer:public Mac802_16Timer{
	public:
		SendDCDtimer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double);	
		void 	stop();	
		void	handle(Event *e);
};
class SendUCDtimer:public Mac802_16Timer{
	public:
		SendUCDtimer(Mac802_16* m):Mac802_16Timer(m){};
		void 	start(double);	
		void 	stop();	
		void	handle(Event *e);
};
#endif /*__802_16timer_h__*/
